<?php
  /**
   * Content Assistant
   *
   * A kind of Headless CMS system build over the Codeigniter framework (http://codeigniter.com)
   *
   * @package   ContentAssistant
   * @author    A59327424 @ Codeinalab
   * @copyright 
   * @license   
   * @link      
   * @version   0.10.0
   */

  /**
   * Ca_authentication model
   *
   * User authentication
   *
   * @todo Integrate Google/Facebook authentication
   */
	class Ca_authentication extends CI_Model
	{
		public $result_code;
		public $result_message;
		
		public $username;
		private $password;
		public $user_id;
		public $ip;
		private $authorized = false;
		public $session;

		public function __construct() {
    		parent::__construct();
    		if ($this->db->table_exists(config_item('ca_tables_prefix') . 'sessions'))
    		{
    			$this->validate();
    		}
		}

		/**
		 * Init
		 * 
		 * Set object's basic properties
		 * @param string $username
		 * @param string $password
		 */
		function set($username, $password) 
		{
			$this->username = $username;
			$this->password = $this->encryption->encrypt($password);
			$this->ip = $this->input->ip_address();
		}

		/**
		 * Authorize authentication session
		 *
		 * Verify credentials against DB
		 * Using php password_hash and CI encryption library
		 * Updates object's result code property
		 * Result codes (0=>Query error, 1=>Success, 2=>User does not exist, 3=>Wrong password, 4=>Account not verified, 5=>Account disabled)
		 * 
		 * @return boolean
		 */
		function authorize() 
		{
			$query_params = array('email' => $this->username);
			$query = $this->db->get_where(config_item('ca_tables_prefix') . 'users', $query_params);
			if ($query)
			{
				if (1 == $query->num_rows())
				{
					foreach ($query->result() as $row)
					{
						$this->user_id = $row->id;
						if (1 == $row->status_id)
						{
							// Bienvenido-Welcome
							if (password_verify($this->encryption->decrypt($this->password), $row->password))
							{
								$this->authorized = true;
								$this->result_code = 1;
								return true;
							}
							else
							{
								// Wrong password
								$this->result_code = 3;
								return false;
							}
						}
						elseif (2 == $row->status_id)
						{
							// Unverified account
							$this->result_code = 4;
							return false;
						}
						elseif (0 == $row->status_id || 3 <= $row->status_id)
						{
							// Disabled
							$this->result_code = 5;
							return false;
						}
				}
			}
			else
			{
				// Username does not exist
				$this->result_code = 2;
				return false;
				}
			}
			else
			{
				$this->result_code = 0;
				return false;
			}
		}

		/**
		 * Log authentication session
		 *
		 * Register authentication session data on DB, _SESSION and _COOKIE 
		 * if session is_persistent
		 *
		 * @param  boolean $is_persistent
		 * 
		 * @return boolean
		 */
		function log($is_persistent) 
		{
			$project_name = $this->ca_utility->settings_get(null, 'project')[0]['items']['name']['value'];
			$f_project_name = friendly_string_get($project_name);
			$this->session = random_string('alnum', 24);
			$encrypted_session = $this->encryption->encrypt($this->session);
			$encrypted_user_id = $this->encryption->encrypt($this->user_id);
			if ($is_persistent == 1)
			{
				$server_name = $_SERVER['SERVER_NAME'];
				$domain = '';
				if ($server_name != 'localhost')
				{
					$domain = str_replace('www', '', $server_name);
				}
				// Set cookie
				$this->load->helper('cookie');
				$cookie_name = $f_project_name . "_data";
				$cookie_data = "" . $encrypted_session . '_' . $encrypted_user_id . "";
				// Set expiration date to 5 days from start, 86400 = 1 day (60 * 60 * 24)
				$cookie_exp = time() + (86400 * 5);
				//setcookie($cookie_name, $cookie_data, $cookie_exp, "/", "" . $domain . "");
				set_cookie($cookie_name, $cookie_data, $cookie_exp, $domain, '/');
			}
			$now = date('Y-m-d H:i:s');
			$query_data = array('user_id' => $this->user_id, 
													'session' => $this->session, 
													'ip' => $this->ip, 
													'start' => $now, 
													'status_id' => 1, 
													'is_persistent' => $is_persistent);
			$query = $this->db->insert(config_item('ca_tables_prefix') . 'sessions', $query_data);
			if (1 == $this->db->affected_rows())
			{
				$user_data = array('session'=>$encrypted_session, 
								   				 'user_id'=>$encrypted_user_id);
				$_SESSION[$f_project_name . '_data'] = $user_data;
				$this->result_code = 1;
				return true;
			}
			else
			{
				$this->result_code = 0;
				return false;
			}
		}

		/**
		 * Validate authentication sessions
		 *
		 * Check authentication session data from _SESSION/_COOKIE against DB
		 * 
		 * @return boolean
		 */
		function validate() 
		{
			$project_name = $this->ca_utility->settings_get(null, 'project')[0]['items']['name']['value'];
			$f_project_name = friendly_string_get($project_name);
			// Check for authentication session data in _COOKIE
			if (isset($_COOKIE[$f_project_name . '_data']))
			{
				$session_data = explode('_', $_COOKIE[$f_project_name . '_data']);
				$session = $this->encryption->decrypt($session_data[0]);
				$user_id = $this->encryption->decrypt($session_data[1]);
				// Check against DB
				$query_params = array('user_id' => $user_id, 
														  'session' => $session,
														  'is_persistent' => 1,
														  'status_id' => 1);
				$query = $this->db->get_where(config_item('ca_tables_prefix') . 'sessions', $query_params);
				(1 == $query->num_rows()) ? $this->authorized = true : $this->authorized= false;
				// Register authentication session data in _SESSION
				$user_data = array('session'=>$this->encryption->encrypt($session), 
													 'user_id'=>$this->encryption->encrypt($user_id));
				$_SESSION[$f_project_name . '_data'] = $user_data;
			}
			// Check for authentication session data in _SESSION
			elseif (isset($_SESSION[$f_project_name . '_data']))
			{
				$session = $this->encryption->decrypt($_SESSION[$f_project_name . '_data']['session']);
				$user_id = $this->encryption->decrypt($_SESSION[$f_project_name . '_data']['user_id']);
				// Check against DB
				$query_params_where = array('user_id' => $user_id, 
																	  'session' => $session,
																	  'status_id' => 1);
				$query = $this->db->get_where(config_item('ca_tables_prefix') . 'sessions', $query_params_where);
				(1 == $query->num_rows()) ? $this->authorized = true : $this->authorized = false;
			}
			return $this->authorized;
		}

		/**
		 * Check authentication
		 *
		 * Returns object private value
		 * 
		 * @return boolean
		 */
		function check() 
		{
			return $this->authorized;
		}
		
		/**
		 * End authentication session
		 *
		 * Closes the active session
		 * Updates object's result code property
		 * Result codes (0=>Query error, 1=>Success, 2=>No session to end)
		 * 
		 * @return boolean
		 */
		function end() 
		{
			$project_name = $this->ca_utility->settings_get(null, 'project')[0]['items']['name']['value'];
			$f_project_name = friendly_string_get($project_name);
			if (isset($_SESSION[$f_project_name . '_data']))
			{
				$session = $this->encryption->decrypt($_SESSION[$f_project_name . '_data']['session']);
				$user_id = $this->encryption->decrypt($_SESSION[$f_project_name . '_data']['user_id']);
				$now = date('Y-m-d H:i:s');
				$query_params = array('status_id' => 0, 'end' => $now);
				$query_params_where = array('user_id' => $user_id, 'session' => $session);
				$this->db->update(config_item('ca_tables_prefix') . 'sessions', $query_params, $query_params_where);
				$this->result_message = $this->db->affected_rows();
				if ($this->db->affected_rows())
				{
					$this->authorized = false;
					unset($_SESSION[$f_project_name . '_data']);
					if (isset($_COOKIE[$f_project_name . '_data']))
					{
						// Unset cookie
						$this->load->helper('cookie');
						$cookie_name = $f_project_name . "_data";
						// Set expiration date to 1 days before today, 86400 = 1 day (60 * 60 * 24)
						$cookie_exp = time() - (86400 * 1);
						//setcookie($cookie_name, "", $cookie_exp);
						delete_cookie($cookie_name);
					}
					$this->result_code = 1;
					return true;
				}
				else
				{
					$this->result_code = 0;
					return false;
				}
			}
			else
			{ 
				$this->result_code = 2;
				return true;
			}
		}
	}