<?php
  defined('BASEPATH') OR exit('No direct script access allowed');
  /**
   * Content Assistant
   *
   * A kind of Headless CMS system build over the Codeigniter framework (http://codeigniter.com)
   *
   * @package   ContentAssistant
   * @author    A59327424 @ Codeinalab
   * @copyright 
   * @license   
   * @link      
   * @version   0.10.0
   */

  /**
   * Ca_page model
   *
   * Pages and blog posts are defined here
   *
   * @todo
   */
	class Ca_page extends CI_Model
	{

		public $result_code;
		public $result_message;

    /**
     * Get page data
     *
     * @param  integer $item_id Unique page ID, optional
     * @param  string  $category_id One or multiple ids in a string to be use on a where in statement eg. 1 or 1,3,2
     * @param  integer $parent_id
     * @param  integer $is_default
     * @param  integer $lang_id
     * @param  integer $status_id
     *
     * @return [type] [description]
     *
     * @todo  reevaluate the is_default parameter use/implementation
     */
    public function get($item_id, $category_id, $parent_id, $is_default, $lang_id, $status_id)
    {
      $this->db->select('t1.*, 
                         t2.name AS category_name, t2.slug AS category_slug, 
                         t3.name AS author_name, t3.last_name AS author_last_name, t3.email AS author_email, 
                         COUNT(DISTINCT t4.id) AS has_child_items');

      // Where
      if ($item_id)
      {
        /* item_id can be a string starting with a number, so
           if it is then add an extra string at the begining as 
           an exception to avoid the mysql automatic type conversion in expression evaluation
           https://dev.mysql.com/doc/refman/5.7/en/type-conversion.html */
        if (!is_numeric($item_id)) { $item_id = 'null'.$item_id; }
        $this->db->where('t1.id', $item_id);
      }
      $categories_id = explode(',', $category_id);
      if ($category_id)
      {
        $this->db->where_in('t1.category_id', $categories_id);
      }
      if ($parent_id)
      {
        $this->db->where('t1.parent_id', $parent_id);
      }
      if ($is_default)
      {
          $this->db->where('t1.parent_id IS NULL', null);
      }
      if ($lang_id)
      {
        $this->db->where('t1.lang_id', $lang_id);
      }
      if ($status_id)
      {
        $this->db->where('t1.status_id', $status_id);
      }
      $this->db->join(config_item('ca_tables_prefix') . 'categories AS t2', 't1.category_id = t2.id', 'left')
               ->join(config_item('ca_tables_prefix') . 'users AS t3', 't1.author_id = t3.id', 'left')
               ->join(config_item('ca_tables_prefix') . 'pages AS t4', 't1.id = t4.parent_id', 'left')
               ->group_by('t1.id')
               ->order_by('t1.name', 'ASC');
      $query = $this->db->get(config_item('ca_tables_prefix') . 'pages AS t1');
      $result = [];
      foreach ($query->result_array() as $row) {
        $row['parents_path'] = $this->parent_ids_get($row['id'], 'pages');
        $row['additionals'] = !empty($row['additionals']) ? unserialize($row['additionals']) : null;
        
        // Images
        $main_image = $row['id'] . '_' . friendly_string_get($row['name']);
        $prefix = $row['id'] . '_';
        $folder = 'pages';
        if ($row['parent_id'] == $this->ca_page->id_by_slug_get(['blog', null, null, null,], 1))
        {
          $folder = 'posts';
        }
        $row['main_image'] = $main_image;
        $all_images = $this->ca_utility->folder_files_get($folder, $prefix, 'jpg');

        // Use array_values to reset array indexes
        $images = array_values(array_diff($all_images, [$main_image]));
        $row['additional_images'] = $images;
        $result[] = $row;
      }
      return $result;
    }

    /**
     * Create page
     *
     * @param  integer $parent_id Unique ID of parent element, can be null
     * @param  integer $author_id Unique ID of user
     * @param  integer $category_id Unique ID of category, can be null
     * @param  string  $meta_title
     * @param  string  $meta_description
     * @param  string  $name
     * @param  string  $content Rich text/html
     * @param  array   $additionals Serialized array
     * @param  string  $slug Uri segment
     * @param  string  $template Name of template (view) file
     * @param  integer $lang_id Default to 1
     * @param  integer $status_id
     *
     * @return boolean
     *
     * @todo  Sanitize the $content element
     */
    public function create($parent_id, $author_id, $category_id, 
                           $meta_title, $meta_description, $name, $content, 
                           $additionals, $slug, $template, $lang_id, $status_id)
    {
      $now = date('Y-m-d H:i:s');
      $query_data = array('parent_id' => $parent_id,
                          'author_id' => $author_id,
                          'category_id' => $category_id, 
                          'meta_title' => $meta_title,
                          'meta_description' => $meta_description,
                          'name' => $name,
                          'content' => $content,
                          'additionals' => $additionals,
                          'slug' => $slug,
                          'template' => $template,
                          'lang_id' => $lang_id,
                          'created' => $now,
                          'modified' => $now,
                          'status_id' => $status_id);
      $query = $this->db->insert(config_item('ca_tables_prefix') . 'pages', $query_data);
      if ($this->db->affected_rows() == 1)
      {
        $this->result_message['item_id'] = $this->db->insert_id();
        $this->result_code = 1;
        return true;
      }
      else
      {
        
        // Error: Query insert
        $this->result_code = 0;
        return false;
      }
    }

  /**
   * Update page
   *
   * @param  integer $id Unique DB identifier
   * @param  integer $parent_id Unique ID of parent element, can be null
   * @param  integer $category_id Unique ID of category, can be null
   * @param  string $meta_title
   * @param  string $meta_description
   * @param  string $name
   * @param  string $content Rich text/html
   * @param  array $additionals Serialized array
   * @param  string $slug Uri segment
   * @param  string $template Name of template (view) file
   * @param  integer $lang_id Default to 1
   * @param  integer $status_id
   *
   * @return boolean
   */
  public function update($id, $parent_id, $category_id, 
                         $meta_title, $meta_description, $name, $content, 
                         $additionals, $slug, $template, $lang_id, $status_id)
    {
      $now = date('Y-m-d H:i:s');
      $query_data = array('parent_id' => $parent_id,
                          'category_id' => $category_id, 
                          'meta_title' => $meta_title,
                          'meta_description' => $meta_description,
                          'name' => $name,
                          'content' => $content,
                          'additionals' => $additionals,
                          'slug' => $slug,
                          'template' => $template,
                          'lang_id' => $lang_id,
                          'modified' => $now,
                          'status_id' => $status_id);
      $query_params_where = array('id' => $id);
      $query = $this->db->update(config_item('ca_tables_prefix') . 'pages', $query_data, $query_params_where);
      if ($this->db->affected_rows() == 1)
      {
        $this->result_message['item_id'] = $id;
        $this->result_code = 1;
        return true;
      }
      else
      {

        // Error: Query update
        $this->result_code = 0;
        return false;
      }
    }

		/**
     * Get Page ID by slug
     *
     * @param  array   $params  URI segments
     * 
     * @return integer Item ID, returns 404 ID if slug not found
     *
     * @todo  Replace hardcoded URIs with language variables for blog/catalogue default paths
     */
    public function id_by_slug_get($params, $status_id)
		{
      $slug = $params[0];
      if (!empty($params[1]))
      {
        $slug.= '/' . $params[1];
      }
      $table = ('catalogo/producto' == $slug) ? 'products' : 'pages';
      // Blog/Catalogue categories exceptions
      if ('blog/categoria' == $slug OR 
          'catalogo/categoria' == $slug)
      {
        $slug = $params[0];
      }
      if ('blog/post' == $slug OR 
          'catalogo/producto' == $slug)
      {
        $slug.= '/' . $params[2];
      }
			$this->db->select('id')->where('slug', $slug);
      if (!empty($status_id))
      {
        if (1 === $status_id)
        {
          $this->db->where('status_id <>', 0);
        }
        else
        {
          $this->db->where('status_id', $status_id);
        }
      }
			$query = $this->db->limit(1)
												->get(config_item('ca_tables_prefix') . $table)
												->row_array();
			if (empty($query))
      {
        $id = $this->id_by_slug_get(['404', null, null], 1);
      }
      else
      {
        $id = $query['id'];
      }
      return $id; 
		}

    /**
     * Get Category ID by slug
     *
     * @param  string  $slug URI
     * @param  integer $status_id
     *
     * @return integer
     */
    public function category_id_by_slug_get($slug, $status_id)
    {
      $query = $this->db->select('id')
                        ->where('slug', $slug)
                        ->where('status_id', $status_id)
                        ->limit(1)
                        ->get(config_item('ca_tables_prefix') . 'categories')
                        ->row_array();
      $id = (empty($query)) ? 0 : $query['id'];
      return $id;
    }

    /**
     * Get categories
     *
     * @param  integer $item_id Unique DB identifier
     * @param  integer $parent_id Unique ID of parent element, can be null
     * @param  integer $lang_id
     * @param  integer $status_id
     *
     * @return array
     */
    public function category_get($item_id, $parent_id, $lang_id, $status_id)
    {

      $this->db->select('t1.*, t2.name AS parent_name, 
                         (COUNT(t3.category_id) + COUNT(t4.category_id)) AS has_items, 
                         COUNT(DISTINCT t5.id) AS has_child_items');

      // Where
      $query_where = [];
      if ($item_id)
      {
        $query_where['t1.id'] = $item_id;
      }
      $parent_ids = '';
      if ($parent_id)
      {
        $this->db->where_in('t1.parent_id', explode(',', $parent_id));
      }
      if ($lang_id)
      {
        $query_where['t1.lang_id'] = $lang_id;
      }
      if ($status_id)
      {
        $query_where['t1.status_id'] = $status_id;
      }
      $this->db->where($query_where)
               ->join(config_item('ca_tables_prefix') . 'categories AS t2', 't1.parent_id = t2.id', 'left')
               ->join(config_item('ca_tables_prefix') . 'pages AS t3', 't1.id = t3.category_id', 'left')
               ->join(config_item('ca_tables_prefix') . 'products AS t4', 't1.id = t4.category_id', 'left')
               ->join(config_item('ca_tables_prefix') . 'categories AS t5', 't1.id = t5.parent_id', 'left')
               ->group_by('t1.id')
               ->order_by('t1.name', 'ASC');
      $query = $this->db->get(config_item('ca_tables_prefix') . 'categories AS t1');
      $result = [];
      foreach ($query->result_array() as $row) {
        $row['parents_path'] = $this->parent_ids_get($row['id'], 'categories');
        $result[] = $row;
      }
      return $result;
    }

    /**
     * Get Category parent IDs
     *
     * @param  integer $item_id Unique category ID
     *
     * @return string Full parent id's path as a string eg. 1.3.5 
     */
    public function parent_ids_get($item_id, $type)
    {
      $path = $item_id . '.';
      $query = $this->db->select('id, parent_id')
               ->where('id', $item_id)
               ->get(config_item('ca_tables_prefix') . $type);
      foreach ($query->result_array() as $row) {
        if ($row['parent_id'])
        {
          $path = $this->parent_ids_get($row['parent_id'], $type) . $path;
          $this->parent_ids_get($row['parent_id'], $type);
        }
      }
      return $path;
     }

    /**
     * Create Category
     *
     * @param  integer $parent_id Unique ID of parent element
     * @param  string  $name
     * @param  string  $slug URI
     * @param  integer $lang_id
     *
     * @return [type] [description]
     */
    public function category_create($parent_id, $name, $slug, $lang_id)
    {
      $now = date('Y-m-d H:i:s');
      $query_data = array('parent_id' => $parent_id,
                          'name' => $name,
                          'slug' => $slug,
                          'lang_id' => $lang_id,
                          'created' => $now,
                          'modified' => $now);
      $query = $this->db->insert(config_item('ca_tables_prefix') . 'categories', $query_data);
      if ($this->db->affected_rows() == 1)
      {
        $this->result_message['item_id'] = $this->db->insert_id();
        $this->result_code = 1;
        return true;
      }
      else
      {

        // Error: Query insert
        $this->result_code = 0;
        return false;
      }
    }

    /**
     * Update Category
     *
     * @param  integer $item_id Unique ID
     * @param  integer $parent_id Unique ID of parent element
     * @param  string  $name
     * @param  string  $slug URI
     * @param  string  $lang_id
     *
     * @return [type] [description]
     */
    public function category_update($item_id, $parent_id, $name, $slug, $lang_id)
    {
      $now = date('Y-m-d H:i:s');
      $query_data = array('id' => $item_id,
                          'parent_id' => $parent_id, 
                          'name' => $name,
                          'slug' => $slug,
                          'lang_id' => $lang_id,
                          'modified' => $now);
      $query_params_where = array('id' => $item_id);
      $query = $this->db->update(config_item('ca_tables_prefix') . 'categories', $query_data, $query_params_where);
      if ($this->db->affected_rows() == 1)
      {
        $this->result_message['item_id'] = $item_id;
        $this->result_code = 1;
        return true;
      }
      else
      {

        // Error: Query update
        $this->result_code = 0;
        return false;
      }
    }


  }