<?php
  defined('BASEPATH') OR exit('No direct script access allowed');
  /**
   * Content Assistant
   *
   * A kind of Headless CMS system build over the Codeigniter framework (http://codeigniter.com)
   *
   * @package   ContentAssistant
   * @author    A59327424 @ Codeinalab
   * @copyright 
   * @license   
   * @link      
   * @version   0.10.0
   */
  
  /**
   * Ca_user model
   *
   * All user related is build from here
   *
   * @todo 
   */
	class Ca_user extends CI_Model
	{

		public $result_code;
		public $result_message;

		/**
		 * Get
		 *
		 * @param  integer $item_id
		 *
		 * @return boolean
		 */
		function get($item_id) 
		{
			$query_where = [];
			if ($item_id) { $query_where['t1.id'] = $item_id; }
			$query = $this->db->select('t1.id, t1.name, t1.last_name, t1.email, t1.username,
											  	   			t1.gender, t1.dob, t1.telephone, t1.mobile_phone, t1.is_newsletter_subscriber,  
											  	   			t1.role_id,  
											  	   			t1.created, t1.modified, t1.status_id')
												->where($query_where)
												->get(config_item('ca_tables_prefix') . 'users AS t1')
												->result_array();
			return $query;
		}

		
		
		/**
		 * Create user
		 *
		 * Create a new user in DB
		 *
		 * @param  string  $name
		 * @param  string  $last_name
		 * @param  string  $email
		 * @param  string  $password
		 * @param  string  $verification_code
		 * @param  string  $gender
		 * @param  string  $dob
		 * @param  string  $telephone
		 * @param  string  $mobile_phone
		 * @param  boolean $is_newsletter_subscriber
		 * @param  integer $role_id
		 *
		 * @return boolean
		 */
		function create($name, $last_name, $email, $username, $password, 
										$verification_code, 
										$gender, $dob, $telephone, $mobile_phone, 
										$is_newsletter_subscriber, $role_id) 
		{
			if (!$this->is_registered($email, null)) {
				$hashed_password = password_hash($password, PASSWORD_DEFAULT);
				$now = date('Y-m-d H:i:s');
				(null != $verification_code) ? $status_id = 2 : $status_id = 1;
				$query_data = array('name' => $name,
														'last_name' => $last_name,
														'email' => $email,
														'username' => $username,
														'password' => $hashed_password,
														'verification_code' => $verification_code,
														'gender' => $gender,
														'dob' => $dob,
														'telephone' => $telephone,
														'mobile_phone' => $mobile_phone,
														'is_newsletter_subscriber' => $is_newsletter_subscriber,
														'role_id' => $role_id,
														'created' => $now,
														'modified' => $now,
														'status_id' => $status_id);
				$query = $this->db->insert(config_item('ca_tables_prefix') . 'users', $query_data);
				if ($this->db->affected_rows() == 1)
				{
					$user_id = $this->db->insert_id();
					$this->result_message['user_id'] = $user_id;
					$this->result_code = 1;
					return true;
				}
				else
				{
					// Error: Query insert
					$this->result_code = 0;
					return false;
				}
			}
			else
			{
				// Error: Duplicated email
				$this->result_code = 2;
				return false;
			}
		}

		/**
		 * Update user
		 *
		 * @param  integer $user_id
		 * @param  string  $name
		 * @param  string  $last_name
		 * @param  string  $email
		 * @param  string  $username
		 * @param  string  $gender
		 * @param  string  $dob
		 * @param  string  $telephone
		 * @param  string  $mobile_phone
		 * @param  boolean $is_newsletter_subscriber
		 * @param  integer $role_id
		 *
		 * @return boolean
		 */
		function update($user_id, $name, $last_name, $email, $username, 
									  $gender, $dob, $telephone, $mobile_phone, 
										$is_newsletter_subscriber, $role_id) 
		{
			if (!$this->is_registered($email, $user_id))
			{
				$now = date('Y-m-d H:i:s');
				$query_data = array('name' => $name,
									'last_name' => $last_name,
									'email' => $email,
									'username' => $username, 
									'gender' => $gender,
									'dob' => $dob,
									'telephone' => $telephone,
									'mobile_phone' => $mobile_phone,
									'is_newsletter_subscriber' => $is_newsletter_subscriber,
									'role_id' => $role_id,
									'modified' => $now);
				$query_params_where = array('id' => $user_id);
				$query = $this->db->update(config_item('ca_tables_prefix') . 'users', $query_data, $query_params_where);
				if ($this->db->affected_rows() == 1)
				{
					$this->result_code = 1;
					return true;
				}
				else
				{
					// Error: Query insert
					$this->result_code = 0;
					return false;
				}
			}
			else
			{
				// Error: Duplicated email
				$this->result_code = 2;
				return false;
			}
		}

		/**
		 * Is registered
		 * 
		 * Check if an email is already registered, 
		 * Use user_id to create an exception when a user changes is own email
		 *
		 * @param  string  $email
		 * @param  integer $user_id
		 *
		 * @return mixed   user id on success false otherwise
		 */
		function is_registered($email, $user_id) 
		{
			
			$query_where = [];
			$query_where['email'] = $email;
			if ($user_id)
			{ 
				$query_where['id'] = '!='.$user_id;
			}
			$query = $this->db->select('id')
												->where($query_where)
												->limit(1)
												->get(config_item('ca_tables_prefix') . 'users')
												->row_array();
			$result = ($query) ? $query['id'] : false;
			return $result;
		}

		/**
		 * Password recover
		 *
		 * Creates a reset_code to be used as a temporary password 
		 *
		 * @param  string $email
		 *
		 * @return mixed false if user does not exists, reset code as a string is success
		 */
		function password_recover($email) 
		{
			if (!$this->is_registered($email, null))
			{
				$this->result_code = 2;
				return false;
			}
			else
			{
				$now = date('Y-m-d H:i:s');
				$reset_code = random_string('alnum', 16);
				$encrypted_reset_code = $this->encryption->encrypt($reset_code);
				$hashed_reset_code = password_hash($reset_code, PASSWORD_DEFAULT);
				$query_update = array('reset_code' => $hashed_reset_code,
										  				'modified' => $now);
				$query_where = array('email' => $email);
				$query = $this->db->update(config_item('ca_tables_prefix') . 'users', $query_update, $query_where);
				$this->result_code = 1;
				$this->result_message['reset_code'] = $encrypted_reset_code;
				return $encrypted_reset_code;
			}
		}

		/**
		 * Password reset
		 *
		 *  Result codes: (0->Query error: Update, 1->Success, 2->Wrong current password)
		 *
		 * @param  string $password
		 * @param  string $reset_code
		 *
		 * @return boolean
		 */
		function password_reset($reset_code, $password) 
		{
			$decrypted_reset_code = $this->encryption->decrypt($reset_code);
			$query_where = [];
			$query_where['reset_code !='] = null;
			$query_where['modified >'] = date('Y-m-d H:i:s', strtotime('-15 minutes'));
			$query = $this->db->select('id, reset_code')
												->where($query_where)
												->get(config_item('ca_tables_prefix') . 'users')
												->result_array();
			foreach ($query as $row) 
			{
				if (password_verify($decrypted_reset_code, $row['reset_code']))
				{
					if (!$this->ac_password_update($row['id'], $password))
					{
						$this->result_code = 2;
						return false;
					}
					$this->result_code = 1;
					return $row['id'];
				}
			}
			$this->result_code = 0;
			return false;

		}

		/**
		 * Password update
		 *
		 * User update his own password, verifying current password
		 * Result codes: (0->Query error: Update, 1->Success, 2->Wrong current password)
		 *
		 * @param  integer $user_id
		 * @param  string  $user_email
		 * @param  string  $current_password
		 * @param  string  $new_password
		 *
		 * @return boolean
		 */
		function password_update($user_id, $user_email, 
														 $current_password, $new_password) 
		{
			// Check current password
			$this->ca_authentication->set($user_email, $current_password);
			if ($this->ca_authentication->authorize())
			{
				if ($this->ac_password_update($user_id, $new_password))
				{
					$this->result_code = 1;
					return true;
				}
				else
				{
					// Error: Query update
					$this->result_code = 0;
					return false;	
				}
			}
			else
			{
				// Error: wrong current password
				$this->result_code = 2;
				return false;
			}
		}
		
		/**
		 * Admin passwor update
		 *
		 * Password update for admins, 
		 * updates users passwords without current password verification
		 *
		 * @param  integer $user_id
		 * @param  string  $password
		 *
		 * @return boolean
		 */
		public function ac_password_update($user_id, $password)
    {

      $now = date('Y-m-d H:i:s');
      $hashed_password = password_hash($password, PASSWORD_DEFAULT);
      $query_where = [];
      $query_where['id'] = $user_id;
      $query_data = array('password' => $hashed_password,
                          'reset_code' => null, 
                          'modified' => $now);
      $query = $this->db->update(config_item('ca_tables_prefix') . 'users', $query_data, $query_where);
      if ($this->db->affected_rows() == 1)
      {
        $this->result_code = 1;
        return true;
      }
      else
      {
        $this->result_code = 0;
        return false;
      }

    }


	}