<?php
  defined('BASEPATH') OR exit('No direct script access allowed');
  /**
   * Content Assistant
   *
   * A kind of Headless CMS system build over the Codeigniter framework (http://codeigniter.com)
   *
   * @package   ContentAssistant
   * @author    A59327424 @ Codeinalab
   * @copyright 
   * @license   
   * @link      
   * @version   0.10.0
   */
  
  /**
   * Ca_utility model
   *
   * A collection of functions that can be used by multiple models/controllers
   *
   * @todo 
   */
  class Ca_utility extends CI_Model
  {

    public $result_code;
    public $result_message;

    /**
     * Get Ca settingsg from DB
     *
     * Obtain the Ca setting items from settings table either using name or id
     * items are a multidimensional array (serialized in db)
     *
     * @param  integer $item_id Unique DB identifier
     * @param  string  $name String identifier
     *
     * @return array The settings items in array
     */
    public function settings_get($item_id, $name)
    {
      $query_where = [];
      if ($item_id)
      {
        $query_where['id'] = $item_id;
      }
      if ($name)
      {
        $query_where['name'] = $name;
      }
      $query = $this->db->select('*')
                        ->where($query_where)
                        ->get(config_item('ca_tables_prefix') . 'settings');
      $result = null;
      foreach ($query->result_array() as $row)
      {
        $row['items'] = unserialize($row['items']);
        $result[] = $row;
      }
      return $result;
    }

    /**
     * Add Ca settings
     *
     * Create a new setting item in DB
     *
     * @param  string $name String identifier
     * @param  arrary $items
     *
     * @return boolean If true new item id is set in the result_code propoerty
     */
    public function settings_add($name, $items)
    {
      $now = date('Y-m-d H:i:s');
      $query_data = array('name' => $name,
                          'items' => $items,
                          'created' => $now,
                          'modified' => $now);
      $query = $this->db->insert(config_item('ca_tables_prefix') . 'settings', $query_data);
      if ($this->db->affected_rows() == 1)
      {
        $this->result_message['item_id'] = $this->db->insert_id();
        $this->result_code = 1;
        return true;
      }
      else
      {
        // Error: Query insert
        $this->result_code = 0;
        return false;
      }
    }

    /**
     * Update Ca settings
     *
     * Update names and values from existing records
     *
     * @param  integer $item_id Unique db identifier
     * @param  string  $name String identifier
     * @param  array   $items
     *
     * @return array
     */
    public function settings_update($item_id, $name, $items)
    {
      $now = date('Y-m-d H:i:s');
      $query_data = array('name' => $name,
                          'items' => $items,
                          'modified' => $now);
      $query_params_where = array('id' => $item_id);
      $query = $this->db->update(config_item('ca_tables_prefix') . 'settings', $query_data, $query_params_where);
      if ($this->db->affected_rows() == 1)
      {
        $this->result_code = 1;
        return true;
      }
      else
      {
        // Error: Query update
        $this->result_code = 0;
        return false;
      }
    }

    /**
     * Status update
     *
     * Update the status id for any item in any of the tables available
     * Table name in params does not include the Ca tables prefix (see config/ca_config.php)
     *
     * @param  string  $table Table name
     * @param  integer $item_id Unique DB identifier
     * @param  integer $status_id 
     *
     * @return [type] [description]
     */
    public function status_update($table, $item_id, $status_id)
    {
      $now = date('Y-m-d H:i:s');
      $query_data = array('status_id' => $status_id,
                          'modified' => $now);
      $query_params_where = array('id' => $item_id);
      $query = $this->db->update(config_item('ca_tables_prefix') . $table, $query_data, $query_params_where);
      if ($this->db->affected_rows() == 1)
      {
        $this->result_code = 1;
        return true;
      }
      else
      {
        $this->result_code = 0;
        return false;
      }
    }

    /**
     * Item delete
     *
     * Delete the specified item in the specified table
     *
     * @param  string $table Table name
     * @param  integer $item_id Unique DB identifier
     *
     * @return boolean
     */
    public function item_delete($table, $item_id)
    {
      $this->db->where('id', $item_id);
      $this->db->delete(config_item('ca_tables_prefix') . $table);
      $this->db->limit(1);
      if ($this->db->affected_rows() === 1)
      {
        $this->result_code = 1;
        return true;
      }
      else
      {
        $this->result_code = 0;
        return false;
      }
    }

		/**
     * Image upload
     *
     * @param  string $file File input
     * @param  string $name Image name
     * @param  string $folder [pages|posts|products]
     *
     * @return boolean
     */
    public function image_upload($file, $name, $folder)
    {
      $path = ($folder) ? './img/uploads/' . $folder . '/' : './img/uploads/';
      $options = [];
      $options['upload_path'] = $path;
      $options['allowed_types'] = 'jpg|jpeg|png';
      $options['max_size'] = 2048;
      $options['max_width'] = 0;
      $options['max_height'] = 0;
      $options['file_ext_tolower'] = true;
      $options['overwrite'] = true;
      $options['file_name'] = $name;
      $this->load->library('upload');
      $this->upload->initialize($options);

      // Check/create directories
      if (!is_dir('./img/uploads'))
      {
        mkdir('./img/uploads', 0755, true);
      }
      if (!is_dir('./img/uploads/' . $folder))
      {
        mkdir('./img/uploads/' . $folder, 0755, true);
      }

      // Upload the image as is
      if ($this->upload->do_upload($file))
      {
        $this->result_message = $this->upload->data();
        $ext = $this->result_message['file_ext'];

        /* Get the [pages|posts|products]_image_sizes from DB and 
         * create a resized copy from the original and add a suffix as identifier */
        $image_sizes = $this->ca_utility->settings_get(null, $folder . '_image_sizes')[0]['items'];

        // Default size
        $default_size = explode(',', $image_sizes['default']['value']);
        if (!$this->image_resize($name, $ext, $folder, $default_size, '_default'))
        {
          $this->result_code = 2;
          return false;
        }

        // Thumbnail
        $thumb_size = explode(',', $image_sizes['thumb']['value']);
        if (!$this->image_resize($name, $ext, $folder, $thumb_size, '_th'))
        {
          $this->result_code = 2;
          return false;
        }

        // Width breakpoints
        $width_breakpoints = explode(',', $image_sizes['width_breakpoints']['value']);
        if (!empty($width_breakpoints))
        {
          $arr = $width_breakpoints;
          for ($i = 0; $i < count($arr); $i++) {
            $breakpoint_size = array(trim($arr[$i]), 0);
            $this->image_resize($name, $ext, $folder, $breakpoint_size, '_' . trim($arr[$i]));
          }
          if ($i != (count($arr)))
          {
            $this->result_code = 2;
            return false;
          }
        }
        $this->result_code = 1;
        return true;
      }
      else
      {
        $this->result_message = $this->upload->data();
        $this->result_code = 0;
        return false;
      }
    }

    /**
     * Image resize
     *
     * Use the Zebra image library instead of codeigniters library because 
     * the crop_cenetr option
     *
     * @param  string  $name Image name
     * @param  string  $ext  Image extension
     * @param  string  $folder [pages|posts|products]
     * @param  array   $image_size eg. (w=>250, h=>250)
     * @param  string  $name_suffix If is set function will create a resized copy of the original image
     *
     * @return boolean
     */
    public function image_resize($name, $ext, $folder, $img_size, $name_suffix)
    {
      $path = ($folder) ? FCPATH . 'img/uploads/' . $folder . '/' : FCPATH . 'img/uploads/';
      if (empty($name_suffix))
      {
        $name_suffix = '';
      }
      $img = new Zebra_Image();
      // $img->preserve_aspect_ratio = true;
      $img->enlarge_smaller_images = false;
      $img->source_path = $path . $name . $ext;
      $img->target_path = $path . $name . $name_suffix . '.jpg';
      $img->resize($img_size[0], $img_size[1], ZEBRA_IMAGE_CROP_CENTER);
      if ($img)
      {
        $this->result_code = 1;
        return true;
      }
      else
      {
        $this->result_code = 0;
        return false;
      }
    }

    /**
     * Folder files get
     *
     * @param  string $type pages|posts|products
     * @param  string $prefix Filename prefix
     * @param  string $ext File extension
     *
     * @return array  List of files available
     */
    public function folder_files_get($type, $prefix, $ext)
    {
      $path = './img/uploads/' . $type . '/';
      $files = (directory_map($path, 1)) ? directory_map($path, 1) : [];
      //* 
      $suffix_filter = '_th|_default|';
      $suffix_filter.= str_replace(',', '|', $this->ca_utility->settings_get(null, $type . '_image_sizes')[0]['items']['width_breakpoints']['value']);
      $pattern = '/^(?=' . $prefix . ')(?:(?!' . $suffix_filter . ').)*(\.' . $ext . ')$/';
      
      $files = array_values(array_filter($files, function($file) use ($pattern)
      {
        return preg_match($pattern, $file);
      }));

      for ($i = 0; $i < count($files); $i++) {
        $files[$i] = pathinfo($files[$i], PATHINFO_FILENAME);
      }

      return $files;
    }


	}